SystemOrganization addCategory: #'SWE-IssueTracker'!

TestCase subclass: #ITTestGlorpDB
	instanceVariableNames: 'login accessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestGlorpDB methodsFor: 'testing' stamp: 'afk 12/13/2007 14:52'!
setUp
	
	super setUp.
	login := ITGlorpDB login.
	accessor := DatabaseAccessor forLogin: login.! !

!ITTestGlorpDB methodsFor: 'testing' stamp: 'mt 12/14/2007 15:47'!
testConnect
	"Tests whether ITGlorpDB>>#connect answers with a valid GlorpSession which could be used for operating with the SQL database."
	
	| session |
	session := ITGlorpDB connect.
	
	self assert: (login username = session currentLogin username) description: 'Username of session does not equal login username'.
	self assert: (login connectString = session currentLogin connectString) description: 'ConnectString of session does not equal login ConnectString'.
	self assert: (login password = session currentLogin password) description: 'Password of session does not equal login password '.
	
	self assert: session isLoggedIn.! !

!ITTestGlorpDB methodsFor: 'testing' stamp: 'mt 12/14/2007 15:44'!
testLogin
	"Tests whether it is possible to use the default login for connecting to a vaild SQL database which is used in the whole application."
	
	self assert: accessor isLoggedIn not.
	accessor loginIfError: [:exception | self assert: false.].
	self assert: accessor isLoggedIn.
	accessor logout.
	self assert: accessor isLoggedIn not.! !

!ITTestGlorpDB methodsFor: 'testing' stamp: 'mig 12/20/2007 18:53'!
testUniqueSession
	"Tests whether a glorp session really is a singleton."


	self assert: ITGlorpSession getInstance notNil description: 'Glorp session is nil.'.
	self assert: (ITGlorpSession getInstance == ITGlorpSession getInstance) description: 'Glorp session is not unique.'.! !

TestCase subclass: #ITTestIssue
	instanceVariableNames: 'issue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/14/2007 16:30'!
setUp

	issue := ITIssue new.! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/14/2007 16:33'!
testAddComment
	
	issue addComment: 'TESTCOMMENT'.
	
	self 
		assert: issue comments last content = 'TESTCOMMENT' 
		description: 'A just added comment could not be retrieved from the issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/14/2007 16:32'!
testComments
	"Tests whether the accessors for #comments do what they should do - set and get."
	
	| comments |
	comments := OrderedCollection new.
	issue comments: comments.
	
	self 
		assert: issue comments == comments 
		description: 'An ordered collection of comments was just set and could not be retrieved from the issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/14/2007 16:32'!
testDetails
	"Tests whether the accessors for #details do what they should do - set and get."
	
	| details |
	details := OrderedCollection new.
	issue details: details.
	
	self 
		assert: issue details == details 
		description: 'An ordered collection of details was just set and could not be retrieved from the issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/14/2007 16:32'!
testGetTag

	| detail |
	detail := ITIssueDetail new.
	detail tag: 'title'.
	detail content: 'TESTTITLE'.
	issue details: detail asOrderedCollection.
	
	self 
		assert: (issue getTag: 'title') = 'TESTTITLE' 
		description: 'A just added detail could not be retrieved from the issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/14/2007 16:30'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."

	issue id: 1.
	
	self
		assert: issue id = 1 
		description: 'An id was just set and could not be retrieved from the issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/14/2007 16:31'!
testSetDetailWithTo
	
	issue setDetailWith: 'title' to: 'TESTTITLE'.
	
	self 
		assert: (issue getTag: 'title') = 'TESTTITLE' 
		description: 'A new detail was set and could not be retrieved from the issue.'! !

TestCase subclass: #ITTestIssueComment
	instanceVariableNames: 'comment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestIssueComment methodsFor: 'testing' stamp: 'mt 12/14/2007 16:28'!
setUp

	comment := ITIssueComment new.! !

!ITTestIssueComment methodsFor: 'testing' stamp: 'mt 12/14/2007 16:29'!
testContent
	"Tests whether the accessors for #content do what they should do - set and get."

	comment content: 'TESTCOMMENT'.
	
	self 
		assert: comment content = 'TESTCOMMENT' 
		description: 'A content was just set and could not be retrieved from the comment.'! !

!ITTestIssueComment methodsFor: 'testing' stamp: 'mt 12/14/2007 16:29'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."

	comment id: 1.
	
	self 
		assert: comment id = 1 
		description: 'An id was just set and could not be retrieved from the comment.'! !

TestCase subclass: #ITTestIssueDetail
	instanceVariableNames: 'detail'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestIssueDetail methodsFor: 'testing' stamp: 'mt 12/14/2007 16:25'!
setUp
	detail := ITIssueDetail new.! !

!ITTestIssueDetail methodsFor: 'testing' stamp: 'mt 12/14/2007 16:28'!
testContent
	"Tests whether the accessors for #content do what they should do - set and get."

	detail content: 'TESTCONTENT'.
	
	self 
		assert: detail content = 'TESTCONTENT' 
		description: 'A content was just set and could not be retrieved from the detail.'! !

!ITTestIssueDetail methodsFor: 'testing' stamp: 'mt 12/14/2007 16:26'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."

	detail id: 1.
	
	self 
		assert: detail id = 1 
		description: 'An id was just set and could not be retrieved from the detail.'! !

!ITTestIssueDetail methodsFor: 'testing' stamp: 'mt 12/14/2007 16:27'!
testTag
	"Tests whether the accessors for #tag do what they should do - set and get."

	detail tag: 'TESTTAG'.
	
	self 
		assert: detail tag = 'TESTTAG' 
		description: 'A tag was just set and could not be retrieved from the detail.'! !

TestCase subclass: #ITTestIssueManager
	instanceVariableNames: 'issueManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestIssueManager methodsFor: 'testing' stamp: 'afk 12/15/2007 12:10'!
setUp
	. 
	issueManager := ITIssueManager new.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 12/15/2007 10:47'!
testAddCommentAtID

	| issue |
	issue := issueManager addIssueWithTitle: 'tmp_testAddCommentAtID'.
	
	issueManager addComment: 'tmp_testAddCommentAtID_comment' atID: issue id.
	
	self 
		assert: (issueManager getIssueWithID: issue id) comments last content = 'tmp_testAddCommentAtID_comment'
		description: 'A comment to a new issue was just set and could not be retrieved through an issue manager.'.
	
	issueManager deleteIssueWithID: issue id.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 12/15/2007 10:55'!
testAddIssueWithTitle
	
	| issue |
	issue := issueManager addIssueWithTitle: 'tmp_testAddIssueWithTitle'.
	
	self 
		assert: (issue isKindOf: ITIssue) 
		description: 'ITIssueManager>>#addIssueWithTitle: is not returning an object with type ITIssue'.
		
	self
		deny: (issue id) isNil
		description: 'ITIssueManager>>#addIssueWithTitle: returns an ITIssue object which has no ID.'.
		
	self
		assert: ((issueManager getIssueWithID: issue id) getTag: 'title') = 'tmp_testAddIssueWithTitle'
		description: 'A new issue with a title was just set and the title could not be retrieved through an issue manager.'.
	
	issueManager deleteIssueWithID: issue id.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 12/15/2007 12:51'!
testAddIssueWithTitleWithComment

	| issue |
	issue := issueManager
		addIssueWithTitle: 'tmp_testAddIssueWithTitleWithComment'
		withComment: 'tmp_testAddIssueWithTitleWithComment_comment'.
	
	self 
		assert: (issue isKindOf: ITIssue) 
		description: 'ITIssueManager>>#addIssueWithTitle:WithComment: is not returning an object with type ITIssue'.
		
	self
		deny: (issue id) isNil
		description: 'ITIssueManager>>#addIssueWithTitle:WithComment: returns an ITIssue object which has no ID.'.
		
	self
		assert: ((issueManager getIssueWithID: issue id) getTag: 'title') = 'tmp_testAddIssueWithTitleWithComment'
		description: 'A new issue with a title and a comment was just set and the title could not be retrieved through an issue manager.'.
		
	self
		assert: ((issueManager getIssueWithID: issue id) comments last content) = 'tmp_testAddIssueWithTitleWithComment_comment'
		description: 'A new issue with a title and a comment was just set and the comment could not be retrieved through an issue manager.'.		
	
	issueManager deleteIssueWithID: issue id.
! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 12/15/2007 10:51'!
testDeleteIssueWithID

	| issue |
	
	issue := issueManager addIssueWithTitle: 'tmp_testDeleteIssueWithID'.
	
	issueManager deleteIssueWithID: issue id.
	
	self
		assert: (issueManager getIssueWithID: issue id) isNil
		description: 'A new issue was just added and could not be deleted through an issue manager.'
	

	! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 12/15/2007 10:54'!
testGetIssueWithID
	
	| issue |
	issue := issueManager addIssueWithTitle: 'tmp_testGetIssueWithID'.
	
	self
		assert: ((issueManager getIssueWithID: issue id) getTag: 'title') = 'tmp_testGetIssueWithID'
		description: 'A new issue was just added and could not be retrieved through an issue manager'.
		
	issueManager deleteIssueWithID: issue id.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 12/15/2007 11:04'!
testGetIssues
	
	issueManager getIssues do: [ :issue |
		self 
			assert: (issue isKindOf: ITIssue)
			description: 'ITIssueManager>>#getIssues returns a collection in where not all objects are issues.'.
		
		self
			deny: (issue id) isNil
			description: 'ITIssueManager>>#getIssues returns a collection in where not all issues have an ID.'.
	].
	
	
	! !

Error subclass: #ITUnauthorizedError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITUnauthorizedError commentStamp: 'mig 12/22/2007 16:58' prior: 0!
An ITUnauthorizedError is thrown, when a user has not sufficient privileges to perform an action.!

Object subclass: #ITGlorpDB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITGlorpDB commentStamp: 'mt 12/14/2007 10:22' prior: 0!
I encapsulate the connection to a SQL database, at the moment with PostGreSQL. Feel free to use my class side method #connect to get a new ITGlorpSession instance. But be aware of the connection. You'll have to check whether ITGlorpSession>>#isLoggedIn.!

!ITGlorpDB class methodsFor: 'debug' stamp: 'mt 12/14/2007 10:37'!
closeAllConnections
	ITGlorpSession allInstances do: [:aSession |
		aSession logout.
		Transcript show: 'DEBUG: You closed a Session.']! !

!ITGlorpDB class methodsFor: 'connection' stamp: 'mt 12/15/2007 12:30'!
connect
	"Connects to the database with the given information from login and returns the current (new) session."	
	
	| accessor session system |
	accessor := DatabaseAccessor forLogin: self login.
	accessor login.

	session := GlorpSession new.
	session accessor: accessor.

	system := ITGlorpDescriptorSystem new.
	session system: system.
	system platform: session platform.
	
	^ session
! !

!ITGlorpDB class methodsFor: 'debug' stamp: 'mig 12/16/2007 16:35'!
createTables
	"I'm creating alle relevant tables but only if there are none of them."
	
	| session |
	Transcript show: 'DEBUG: You try to create all tables again.'.
	session := ITGlorpDB connect.
	session inTransactionDo: [session system allTables do: [:each |
			Transcript cr; show: each name; cr.
			session accessor createTable: each ifError: [:error | Transcript show: error messageText]
		]
	].

	session inTransactionDo: [session system allTables do: [:each |
	session accessor createTableFKConstraints: each ifError: [:error | Transcript show: error messageText]		
		]
	].! !

!ITGlorpDB class methodsFor: 'connection' stamp: 'mt 12/14/2007 10:23'!
login
	"The default login can be found here. It is used across the whole issue tracker application. It is also used for unit tests."
	
	^ Login new database: PostgreSQLPlatform new;
		username: 'issuetracker';
		password: 'weloveseasidedontwe';
		connectString: 'theresnoplacelike.homeftp.org:5432_issuetracker'.! !

Object subclass: #ITIssue
	instanceVariableNames: 'id details comments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssue class methodsFor: 'instance creation' stamp: 'afk 12/13/2007 20:12'!
with: aTitle with: aComment

	| newIssue comment |
	newIssue := self new.
	comment := ITIssueComment new.
	
	comment content: aComment.
	
	newIssue comments add: comment.
	newIssue details add: (ITIssueDetail with: 'title' with: aTitle).

	^ newIssue! !

!ITIssue methodsFor: 'changing' stamp: 'mt 12/15/2007 16:20'!
addComment: aString

	self comments add: (ITIssueComment with: aString).! !

!ITIssue methodsFor: 'accessing' stamp: 'mt 11/23/2007 19:55'!
comments
	^comments.! !

!ITIssue methodsFor: 'accessing' stamp: 'mt 12/11/2007 15:23'!
comments: aCollection
	comments := aCollection.! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 12/13/2007 17:03'!
details
	^ details! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 12/13/2007 17:03'!
details: anObject
	details := anObject! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 12/13/2007 18:18'!
getTag: tag
	details do: 
		[:each | 
		each tag = tag
			ifTrue: [^ each content]
		].
	^nil.! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 16:20'!
id
	^ id! !

!ITIssue methodsFor: 'changing' stamp: 'afk 11/12/2007 16:20'!
id: anObject
	id := anObject! !

!ITIssue methodsFor: 'initialize-release' stamp: 'mt 12/15/2007 16:20'!
initialize

	super initialize.
	details := OrderedCollection new.
	comments := OrderedCollection new.

	! !

!ITIssue methodsFor: 'changing' stamp: 'mt 12/15/2007 16:20'!
setDetailWith: aTag to: aValue
	self details add: (ITIssueDetail with: aTag with: aValue)! !

Object subclass: #ITIssueComment
	instanceVariableNames: 'id content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssueComment class methodsFor: 'instance creation' stamp: 'mig 11/30/2007 18:22'!
newWithID: aNumber WithString: aString
	^ self new
		id: aNumber;
		content: aString;
		yourself.! !

!ITIssueComment class methodsFor: 'instance creation' stamp: 'afk 12/13/2007 20:52'!
with: aComment
	^ self new
		content: aComment;
		yourself.! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
content
	^ content! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
content: anObject
	content := anObject! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
id
	^ id! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
id: anObject
	id := anObject! !

!ITIssueComment methodsFor: 'initialize-release' stamp: 'mt 12/14/2007 10:02'!
initialize
	
	super initialize.
	
	content := ''.! !

Object subclass: #ITIssueDetail
	instanceVariableNames: 'id tag content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssueDetail class methodsFor: 'instance creation' stamp: 'mig 12/20/2007 18:51'!
with: aTag with: aValue
	
	| detail |
	detail := self new.
	detail tag: aTag.
	detail content: aValue.
	
	^ detail.! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:54'!
content
	^ content! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:54'!
content: anObject
	content := anObject! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
id
	^ id! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
id: anObject
	id := anObject! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
tag
	^ tag! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
tag: anObject
	tag := anObject! !

Object subclass: #ITIssueManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssueManager methodsFor: 'accessing' stamp: 'mt 12/15/2007 12:34'!
addComment: aString atID: aNumber
	"Adds a new ITIssueComment object with the content 'aString' to the issue with the ID 'aNumber'."

	| issue glorpSession |
	glorpSession := ITGlorpSession getInstance.
	issue := glorpSession readOneOf: ITIssue where: [:each | each id = aNumber].
	
	glorpSession inUnitOfWorkDo: [
		glorpSession register: issue.
		issue addComment: aString.
	].! !

!ITIssueManager methodsFor: 'changing' stamp: 'mt 12/15/2007 12:35'!
addIssueWithTitle: aString
	"Adds a new issue with a detail 'title' to the database. Returns this new issue."
	
	| session issue |
	session := ITGlorpSession getInstance.
	
	issue := ITIssue new.
	issue setDetailWith: 'title' to: aString.
	
	session inUnitOfWorkDo: [
		session register: issue.
	].

	^issue.! !

!ITIssueManager methodsFor: 'changing' stamp: 'mt 12/15/2007 12:50'!
addIssueWithTitle: aTitleString withComment: aCommentString
	"Adds a new issue with a title and an initial comment to the database. Returns this new issue."
	
	| session issue |
	session := ITGlorpSession getInstance.
	issue := ITIssue with: aTitleString with: aCommentString.
	
	session inUnitOfWorkDo: [
		session register: issue.
	].
	
	^issue.! !

!ITIssueManager methodsFor: 'accessing' stamp: 'mt 12/15/2007 12:35'!
deleteIssueWithID: aNumber
	"Tries to delete an issue with the given ID from the database."
	
	| session issue |
	session := ITGlorpSession getInstance.
	
	issue := session readOneOf: ITIssue where: [:each | each id = aNumber].
	
	session delete: issue.
	! !

!ITIssueManager methodsFor: 'accessing' stamp: 'mig 12/23/2007 21:27'!
getIssueWithID: aNumber
	"Returns an ITIssue object where the 'id' equals 'aNumber'."
	| glorpSession |
	
	self flag: #TODO.
	"ITSecurityManager checkPrivilege: #readissue."

	glorpSession := ITGlorpSession getInstance.
	^ glorpSession readOneOf: ITIssue where: [:each | each id = aNumber].
! !

!ITIssueManager methodsFor: 'accessing' stamp: 'mig 12/23/2007 21:27'!
getIssues
	"Returns all ITIssue objects in a collection that are in the database"
	
	self flag: #TODO.
	"ITSecurityManager checkPrivilege: #listissues."

	^ ITGlorpSession getInstance readManyOf: ITIssue.
	! !

Object subclass: #ITRole
	instanceVariableNames: 'id rolename privileges users'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITRole methodsFor: 'accessing' stamp: 'mig 12/17/2007 15:07'!
addPrivilege: aSymbol
	privileges add: aSymbol asString! !

!ITRole methodsFor: 'testing' stamp: 'mig 12/23/2007 21:23'!
hasPrivilege: aSymbol
	^ privileges anySatisfy: [:privilege | privilege = aSymbol].! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/16/2007 16:23'!
id
	^ id! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/16/2007 16:23'!
id: anObject
	id := anObject! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/16/2007 16:49'!
privileges
	^ privileges! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/17/2007 15:09'!
removePrivilege: aSymbol
	privileges remove: aSymbol asString.! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/16/2007 15:53'!
rolename
	^ rolename! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/16/2007 15:53'!
rolename: anObject
	rolename := anObject! !

Object subclass: #ITSecurityManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITSecurityManager class methodsFor: 'testing' stamp: 'mig 12/22/2007 17:00'!
checkPrivilege: aSymbol 
	| currentSession |
	
	currentSession := WACurrentSession value.
	
	(currentSession securityManager hasUser: currentSession user privilege: aSymbol) ifFalse: [
		ITUnauthorizedError signal.
	].

	^ true.! !

!ITSecurityManager class methodsFor: 'hashing' stamp: 'mig 12/22/2007 17:13'!
hashAlgorithm
	^MD5! !

!ITSecurityManager class methodsFor: 'hashing' stamp: 'mig 12/22/2007 17:13'!
hashPassword: aString
	^ (self hashAlgorithm hashMessage: aString) hex.
	! !

!ITSecurityManager class methodsFor: 'hashing' stamp: 'mig 12/22/2007 17:15'!
hashSize
	"hash size is algorithms size * 2 because hashs are stored as hex"
	^ self hashAlgorithm hashSize * 2
	! !

!ITSecurityManager methodsFor: 'traits' stamp: 'mig 12/22/2007 17:22'!
addRole: aRole
	"add an ITRole object to the database"
	| session |
	
	session := ITGlorpSession getInstance.
	
	session inUnitOfWorkDo: [
		session register: aRole.
	].! !

!ITSecurityManager methodsFor: 'traits' stamp: 'mig 12/22/2007 17:22'!
addUser: aUser
	"add an ITUser object to the database"
	| session |
	
	session := ITGlorpSession getInstance.
	
	session inUnitOfWorkDo: [
		session register: aUser.
	].! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mig 12/22/2007 17:22'!
getRole: aString
	"get ITRole object from database"
	| session |
	
	session := ITGlorpSession getInstance.
	^session readOneOf: ITRole where: [:each | each rolename = aString].
	! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mig 12/22/2007 17:21'!
getUser: aString
	"get ITUser object from database"
	| session |
	
	session := ITGlorpSession getInstance.
	^session readOneOf: ITUser where: [:each | each username = aString].
	! !

!ITSecurityManager methodsFor: 'testing' stamp: 'mig 12/23/2007 21:24'!
hasUser: aUser privilege: aSymbol
	"check whether aUser has the aSymbol privilege"
	| user |
	
	user := self userOrDefaultUser: aUser.
	
	user roles do: [:role |
		(role hasPrivilege: aSymbol) ifTrue: [^true].
	].

	^false.! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mig 12/22/2007 17:20'!
userOrDefaultUser: aUser
	"return aUser if not nil, return the default user if nil"
	
	^aUser ifNil: [self getUser: '*DEFAULT']! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mig 12/22/2007 17:21'!
validatePassword: aPassword for: aUsername
	"return whether aUsername and aPassword match"
	| user pwhash |
	
	user := self getUser: aUsername.
	pwhash := self class hashPassword: aPassword.
	
	user ifNil: [^false].
	^ user passwordHash = pwhash.! !

Object subclass: #ITUser
	instanceVariableNames: 'id username passwordHash roles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITUser methodsFor: 'accessing' stamp: 'mig 12/17/2007 15:00'!
addRole: aRole
	roles add: aRole.! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/16/2007 16:26'!
id
	^ id! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/16/2007 16:26'!
id: anObject
	id := anObject! !

!ITUser methodsFor: 'testing' stamp: 'mig 12/17/2007 15:10'!
isInRole: aRole
	roles contains: aRole.! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
passwordHash
	^ passwordHash! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/17/2007 15:10'!
removeRole: aRole
	roles remove: aRole.! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
roles
	^ roles! !

!ITUser methodsFor: 'passwords' stamp: 'mig 12/17/2007 13:36'!
setPassword: aString
	passwordHash := ITSecurityManager hashPassword: aString.! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
username
	^ username! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
username: anObject
	username := anObject! !

WASession subclass: #ITSession
	instanceVariableNames: 'issueManager securityManager glorpSession user'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITSession methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:45'!
issueManager
	issueManager ifNil: [
		issueManager := ITIssueManager new.
	].
	
	^issueManager.! !

!ITSession methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:46'!
securityManager
	securityManager ifNil: [
		securityManager := ITSecurityManager new.
	].
	
	^securityManager.! !

!ITSession methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:46'!
user
	^ user! !

!ITSession methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:46'!
user: anObject
	user := anObject! !

SUComponent subclass: #ITComponentAddComment
	instanceVariableNames: 'comment issueID'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentAddComment class methodsFor: 'instance creation' stamp: 'mig 11/30/2007 19:36'!
newWith: anID
	| newInstance |
	newInstance := self new.
	newInstance issueID: anID.
	^newInstance.! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:11'!
comment
	^ comment! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:11'!
comment: anObject
	comment := anObject! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:36'!
issueID
	^ issueID! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:36'!
issueID: anObject
	issueID := anObject! !

!ITComponentAddComment methodsFor: 'rendering' stamp: 'mig 12/19/2007 13:46'!
renderBox: aString on: html

	html div id: aString; class: 'box'; style: 'display: none; '; with: [
		html div class: 'header'.
		html div
			class: 'content';
			with: [	html form: [
						html textArea on: #comment of: self.
						html break.
						html submitButton
							callback: [
								| issue issueManager |
								issueManager := self session issueManager.
								issue := issueManager getIssueWithID: self issueID.
				
								issueManager addComment: self comment atID: self issueID.
							];
							text: 'Add Comment'.
						]] 
				]! !

!ITComponentAddComment methodsFor: 'rendering' stamp: 'mig 12/19/2007 13:42'!
renderContentOn: html

	html paragraph: [
			html anchor
				onClick: (html effect id: 'toggle'; perform: #toggleSlide);
			with: 'Add Comment'].""
	self renderBox: 'toggle' on: html. 
! !

GlorpSession subclass: #ITGlorpSession
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

ITGlorpSession class
	instanceVariableNames: 'instance'!

!ITGlorpSession commentStamp: 'mt 12/15/2007 10:12' prior: 0!
I'm a usual GlorpSession and control a cache manager, a description system and all unit of works. Every manipulation of the SQL database goes through my "fingers".

Use #inUnitOfWorkDo: for watching over "dirty" objects and change all relevant data in the database. But DO NOT forget to register them with #register inside a unit of work.

Any read of data doesn't have to be embedded in a unit of work. Just use the result of #read:, #readOneOf:, #readManyOf:, #read:where: a. s. o.

If you want to delete an object you doesn't have to use a unit of work either.!

ITGlorpSession class
	instanceVariableNames: 'instance'!

!ITGlorpSession class methodsFor: 'accessing' stamp: 'afk 12/15/2007 13:10'!
getInstance
	"Using singleton pattern to ensure that there is only one instance of ITGlorpSession"
	
	instance ifNil:
		[ instance := ITGlorpDB connect.].
	instance isLoggedIn ifFalse:
		[ instance := ITGlorpDB connect.].
	^ instance! !

!ITGlorpSession class methodsFor: 'instance creation' stamp: 'afk 12/15/2007 12:00'!
new

	^ Exception new messageText: 'Use getInstance because ITGlorpSession is a singleton'.! !

WAComponent subclass: #ITComponentAddIssue
	instanceVariableNames: 'title initialComment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 15:49'!
initialComment
	^ initialComment! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 15:49'!
initialComment: anObject
	initialComment := anObject! !

!ITComponentAddIssue methodsFor: 'rendering' stamp: 'mig 12/10/2007 17:48'!
renderContentOn: html
	html heading 
		level: 1;
		with: 'Add Issue'.
	html form: 
		[
		html text: 'Title'.
			html break.
		html textInput on: #title of: self.
			html break.
		html text: 'Description'.
			html break.
		html textArea on: #initialComment of: self.
			html break.
		html submitButton 
			callback: [
				self session issueManager addIssueWithTitle: self title withComment: self initialComment.
				self answer. 
			];
			value: 'add Issue'.
		html cancelButton
			callback: [self answer];
			value: 'cancel'.
		]! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 15:49'!
title
	^ title! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 15:49'!
title: anObject
	title := anObject! !

WAComponent subclass: #ITComponentDetails
	instanceVariableNames: 'issueID addcomment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentDetails class methodsFor: 'instance creation' stamp: 'mig 11/30/2007 19:36'!
newWith: anID
	|view|
	view := self new.
	view issueID: anID.
	^view.! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:16'!
children
	^ Array with: addcomment.! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:34'!
issueID
	^ issueID! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:34'!
issueID: anObject
	issueID := anObject! !

!ITComponentDetails methodsFor: 'rendering' stamp: 'mig 12/10/2007 17:49'!
renderContentOn: html

	| issue |
	
	issue := self session issueManager getIssueWithID: self issueID.
	addcomment := ITComponentAddComment newWith: self issueID.

	html heading
		level: 2;
		with: 'detailed view of an issue'.
	html div
		with: [
			html text: issue id.
			html break.
			html text: (issue getTag: 'title').
			
			html break; break.
			html table: [
				(issue comments) do: [:comment|
					html tableRow: [ html tableData: [ html text: comment content ] ].

				].
				html tableRow: [ 
					html tableData with: addcomment.
				].
			].
			html anchor
				callback: [self answer];
				with: [html text: 'return to list'].
		]
					
				! !

WAComponent subclass: #ITComponentListIssues
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentListIssues methodsFor: 'rendering' stamp: 'afk 12/13/2007 21:54'!
renderContentOn: html
	| issues |
	issues := self session issueManager getIssues.
	html heading 
		level: 2;
		with: 'List all issues'.
	html form:[
		html table:[
			html tableRow: [
				html tableData: 'id'.
				html tableData: 'title'.
				html tableData: 'details'.
				html tableData: 'delete'.
				].
			issues do: [:issue|
				html tableRow: [
					html tableData: (issue id asString).
					html tableData: (issue getTag: 'title').
					html tableData: [
						html anchor 
							callback: [self call: (ITComponentDetails newWith: issue id)];
							with: [html text: 'detailed view'].
						].
					html tableData: [
						html anchor
							callback: [((self session issueManager) deleteIssueWithID: issue id)];
							with: [html text: 'delete'].
						].
					].
				].
			].
		].! !

WAComponent subclass: #ITComponentLogin
	instanceVariableNames: 'userInput passwordInput'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentLogin methodsFor: 'accessing' stamp: 'mig 12/17/2007 13:42'!
passwordInput
	^ passwordInput! !

!ITComponentLogin methodsFor: 'accessing' stamp: 'mig 12/17/2007 13:42'!
passwordInput: anObject
	passwordInput := anObject! !

!ITComponentLogin methodsFor: 'rendering' stamp: 'mig 12/17/2007 13:58'!
renderContentOn: html
	html form: [
		html text: 'username:'.
		html textInput on: #userInput of: self.
		html break.
		html text: 'password:'.
		html passwordInput on: #passwordInput of: self.
		html break.
		html submitButton callback: [
			| valid |
			valid := self session securityManager validatePassword: self passwordInput for: self userInput.
			valid ifTrue: [
				self session user: (self session securityManager getUser: userInput).
				self answer: true.
			] ifFalse: [
				self inform: 'Wrong username/password combination!!'.
			].
		]; value: 'Login'.
		html cancelButton callback: [
			self answer: false.
		]; value: 'Cancel'.
	].! !

!ITComponentLogin methodsFor: 'accessing' stamp: 'mig 12/17/2007 13:42'!
userInput
	^ userInput! !

!ITComponentLogin methodsFor: 'accessing' stamp: 'mig 12/17/2007 13:42'!
userInput: anObject
	userInput := anObject! !

WAComponent subclass: #ITComponentMenu
	instanceVariableNames: 'listIssues userStatus'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentMenu class methodsFor: 'testing' stamp: 'mt 11/23/2007 18:52'!
canBeRoot
	^true.! !

!ITComponentMenu methodsFor: 'initialize' stamp: 'mig 12/17/2007 14:35'!
children
	^Array with: self listIssues with: self userStatus.! !

!ITComponentMenu methodsFor: 'initialize' stamp: 'mig 12/17/2007 14:35'!
initialize
	super initialize.
	self listIssues: ITComponentListIssues new.
	self userStatus: ITComponentUserStatus new.! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'mt 11/23/2007 19:05'!
listIssues
	^ listIssues! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'mt 11/23/2007 19:05'!
listIssues: anObject
	listIssues := anObject! !

!ITComponentMenu methodsFor: 'rendering' stamp: 'afk 12/19/2007 01:45'!
renderContentOn: html

	html heading
		level: 1;
		with: 'IssueTracker'.
		
	" Clock "
	html paragraph
		script: (html updater
			every: 1 second;
			on: #renderTimeOn: of: self);
		with: [ self renderTimeOn: html ].
		
	html span with: self userStatus.
	html text: ' '.
	html anchor 
		callback: [ self call: ITComponentAddIssue new ]; 
		with: [ html text: 'add issue'].
	html div
		with: self listIssues.! !

!ITComponentMenu methodsFor: 'rendering' stamp: 'mig 12/19/2007 13:51'!
renderTimeOn: html
	html render: Time now print24! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:36'!
userStatus
	^ userStatus! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:36'!
userStatus: anObject
	userStatus := anObject! !

WAComponent subclass: #ITComponentUserRegister
	instanceVariableNames: 'userInput passwordInput1 passwordInput2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
passwordInput1
	^ passwordInput1! !

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
passwordInput1: anObject
	passwordInput1 := anObject! !

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
passwordInput2
	^ passwordInput2! !

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
passwordInput2: anObject
	passwordInput2 := anObject! !

!ITComponentUserRegister methodsFor: 'rendering' stamp: 'mig 12/20/2007 18:37'!
renderContentOn: html
	html div style: 'background: white; padding: 20px;'; with: [
		html form: [
			html text: 'username:'.
			html textInput on: #userInput of: self.
			html break.
			html text: 'password:'.
			html passwordInput on: #passwordInput1 of: self.
			html break.
			html text: 'verify:'.
			html passwordInput on: #passwordInput2 of: self.
			html break.
			html submitButton callback: [
				| user |
				(self session securityManager getUser: self userInput) ifNotNil: [
					self inform: 'Username already exists!!'.
				] ifNil: [
					self passwordInput1 = self passwordInput2 ifFalse: [
						self inform: 'Passwords do not match!!'.
					] ifTrue:	[
						user := ITUser new.
						user 
							username: self userInput;
							setPassword: self passwordInput1.
						self session securityManager addUser: user.
						self answer.]
				].
			]; value: 'Register'.
			html cancelButton callback: [
			self answer.
			]; value: 'Cancel'.
		].
	].! !

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
userInput
	^ userInput! !

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
userInput: anObject
	userInput := anObject! !

WAComponent subclass: #ITComponentUserStatus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentUserStatus methodsFor: 'rendering' stamp: 'mig 12/19/2007 13:49'!
renderContentOn: html
	self session user ifNil: [
		html text: '[ '.
		html anchor
			callback: [ self lightbox: ITComponentLogin new ]; 
			with: [ html text: 'login'].
		html text: ' | '.
		html anchor 
			callback: [ self lightbox: ITComponentUserRegister new ]; 
			with: [ html text: 'register'].
		html text: ' ]'.
	] ifNotNil: [
		html text: '[ Logged in as ', self session user username, ' | '.
		html anchor
			callback: [self session user: nil.];
			with: 'logout'.
		html text: ' ]'.
	].! !

DescriptorSystem subclass: #ITGlorpDescriptorSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITGlorpDescriptorSystem commentStamp: 'mt 12/14/2007 10:23' prior: 0!
I'm the descriptor system that should be used in every new ITGlorpSession. I'm responsible for all mappings between the IT-Classes and the relevant tables on the SQL server.

You should modify #allTableNames and #constructAllClasses whenever you want to add new objects of the domain space for usage here.

#tableNamed<TABLE_NAME>    ... descriptions about the table structure on the server
#descriptorFor<CLASS_NAME> ... defines mappings between classes and tables
#classModelFor<CLASS_NAME> ... defines types (classes) for symbols (attributes) used by mappings

The relational schemes can be found in the comments for any #tableNamed<TABLE_NAME>. Primary keys are underlined, e.g. SCHEME(ATTRIBUTE_A, ATTRIBUTE_B)
                        -----------!

!ITGlorpDescriptorSystem methodsFor: 'accessing' stamp: 'mig 12/16/2007 19:19'!
allTableNames
	"Here are all tables listed which hold all persistent data."
	
	^ #('ISSUE' 'COMMENT' 'DETAIL' 'USERS' 'ROLE' 'USERROLE' 'ROLEPRIVILEGE')! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mt 12/14/2007 09:54'!
classModelForITIssue: aClassModel
	"The unique ID can be up to 1.073.741.823 - should be enough.
	 -> SmallInteger maxVal
	 4-Byte-Integers (signed) are used in the database"
	
	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #comments collection: OrderedCollection  of: ITIssueComment.
	aClassModel newAttributeNamed: #details collection: OrderedCollection  of: ITIssueDetail.! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mt 12/14/2007 09:55'!
classModelForITIssueComment: aClassModel
	"See #classModelForITIssue for explanation of ID."
	
	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #content type: String.! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mt 12/14/2007 09:55'!
classModelForITIssueDetail: aClassModel
	"See #classModelForITIssue for explanation of ID."

	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #tag type: String.
	aClassModel newAttributeNamed: #content type: String.! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mig 12/17/2007 13:29'!
classModelForITRole: aClassModel

	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #rolename type: String.
	aClassModel newAttributeNamed: #privileges collection: Set of: String.
	! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mig 12/17/2007 14:57'!
classModelForITUser: aClassModel

	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #username type: String.
	aClassModel newAttributeNamed: #passwordHash type: String.
	aClassModel newAttributeNamed: #roles collection: Set of: ITRole.
! !

!ITGlorpDescriptorSystem methodsFor: 'accessing' stamp: 'mig 12/16/2007 19:41'!
constructAllClasses
	"Here are all classes listed which are necessary for the mappings."
	
	^ (super constructAllClasses)
		add: ITIssue;
		add: ITIssueComment;
		add: ITIssueDetail;
		add: ITUser;
		add: ITRole;
		yourself! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mt 12/14/2007 09:45'!
descriptorForITIssue: aDescriptor
	"Every issue contains a unique ID, a collection of comments and a collection of details."
	
	| issueTable commentTable detailTable |
	issueTable   := self tableNamed: 'ISSUE'.
	commentTable := self tableNamed: 'COMMENT'.
	detailTable  := self tableNamed: 'DETAIL'.

	aDescriptor table: issueTable.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (issueTable fieldNamed: 'ID').

	(aDescriptor newMapping: ToManyMapping)
		attributeName: #comments;
		join: (Join
			from: (issueTable fieldNamed: 'ID')
			to: (commentTable	fieldNamed: 'ISSUE_ID')
		).
	
	(aDescriptor newMapping: ToManyMapping)
		attributeName: #details;
		join: (Join
			from: (issueTable fieldNamed: 'ID')
			to:  (detailTable fieldNamed: 'ISSUE_ID')
		).! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mt 12/14/2007 09:47'!
descriptorForITIssueComment: aDescriptor
	"Every comment contains a unique ID and its content - should be text."
	
	| table |
	table := self tableNamed: 'COMMENT'.
	aDescriptor table: table.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #content to: (table fieldNamed: 'CONTENT').! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mt 12/14/2007 09:47'!
descriptorForITIssueDetail: aDescriptor
	"Every detail contains a unique ID, a tag and its content."

	| table |
	table := self tableNamed: 'DETAIL'.
	aDescriptor table: table.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #tag to: (table fieldNamed: 'TAG').
	(aDescriptor newMapping: DirectMapping) from: #content to: (table fieldNamed: 'CONTENT').! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mig 12/16/2007 19:26'!
descriptorForITRole: aDescriptor
	
	| roleTable userRoleTable rolePrivilegeTable |
	roleTable := self tableNamed: 'ROLE'.
	userRoleTable := self tableNamed: 'USERROLE'.
	rolePrivilegeTable := self tableNamed: 'ROLEPRIVILEGE'.

	aDescriptor table: roleTable.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (roleTable fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #rolename to: (roleTable fieldNamed: 'ROLENAME').

	(aDescriptor newMapping: ManyToManyMapping)
		attributeName: #users;
		referenceClass: ITUser;
		mappingCriteria: (Join
			from: (roleTable fieldNamed: 'ID')
			to:  (userRoleTable fieldNamed: 'ROLE_ID')
		).
	
	(aDescriptor newMapping: DirectToManyMapping)
		attributeName: #privileges;
		resultField: (rolePrivilegeTable fieldNamed: 'PRIVILEGE');
		writeTheOrderField;
		orderBy: [:each | (each getTable: rolePrivilegeTable) getField: 'PRIVILEGE_ORDER'];
		join: (Join from: (roleTable fieldNamed: 'ID') to: (rolePrivilegeTable fieldNamed: 'ROLE_ID')).! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mig 12/16/2007 19:41'!
descriptorForITUser: aDescriptor
	
	| userTable userRoleTable |
	userTable   := self tableNamed: 'USERS'.
	userRoleTable := self tableNamed: 'USERROLE'.

	aDescriptor table: userTable.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (userTable fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #username to: (userTable fieldNamed: 'USERNAME').
	(aDescriptor newMapping: DirectMapping) from: #passwordHash to: (userTable fieldNamed: 'PASSWORDHASH').

	(aDescriptor newMapping: ManyToManyMapping)
		attributeName: #roles;
		referenceClass: ITRole;
		mappingCriteria: (Join
			from: (userTable fieldNamed: 'ID')
			to:  (userRoleTable fieldNamed: 'USER_ID')
		).! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 12/15/2007 14:29'!
tableForCOMMENT: aTable
	"COMMENTS(ID, ISSUE_ID, TIMESTAMP, CONTENT)
	          --"
	
	| issueId |
	(aTable createFieldNamed: 'ID' type: (platform sequence)) bePrimaryKey.
	issueId := aTable createFieldNamed: 'ISSUE_ID' type: (platform int4).
	aTable createFieldNamed: 'TIMESTAMP' type: (platform timestamp).
	aTable createFieldNamed: 'CONTENT' type: (platform text).
	
	"foreign key definition for the IDs of issues"
	aTable 
		addForeignKeyFrom: issueId 
		to: ((self tableNamed: 'ISSUE') fieldNamed: 'ID')
		suffixExpression: 'ON DELETE CASCADE'.
		! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 12/15/2007 14:28'!
tableForDETAIL: aTable
	"DETAIL(ID, ISSUE_ID, TAG, CONTENT)
	       --"
	
	| issueID |
	(aTable createFieldNamed: 'ID' type: (platform sequence)) bePrimaryKey.
	issueID := (aTable createFieldNamed: 'ISSUE_ID' type: (platform int4)).
	aTable createFieldNamed: 'TAG' type: (platform varchar: 255).
	aTable createFieldNamed: 'CONTENT' type: (platform varchar: 255).
	
	"foreign key definition for IDs of issues"
	aTable
		addForeignKeyFrom: issueID
		to: ((self tableNamed: 'ISSUE') fieldNamed: 'ID')
		suffixExpression: 'ON DELETE CASCADE'.! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 12/14/2007 09:38'!
tableForISSUE: aTable
	"ISSUE(ID)
	       --"
	
	(aTable createFieldNamed: 'ID' type: (platform sequence)) bePrimaryKey.! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mig 12/17/2007 00:50'!
tableForROLE: aTable
	"ROLE(ID, ROLENAME)"
	
	(aTable createFieldNamed: 'ID' type: platform sequence) bePrimaryKey.
	aTable createFieldNamed: 'ROLENAME' type: (platform varchar: 255).! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mig 12/17/2007 14:51'!
tableForROLEPRIVILEGE: aTable
	"ROLEPRIVILEGE(ROLE_ID, PRIVILEGE, PRIVILEGE_ORDER)"
	
	| roleid privilege |
	
	roleid := aTable createFieldNamed: 'ROLE_ID' type: platform int4.
	privilege := aTable createFieldNamed: 'PRIVILEGE' type: platform text.
	aTable createFieldNamed: 'PRIVILEGE_ORDER' type: platform int4.
	
	roleid bePrimaryKey.
	privilege bePrimaryKey.
	
	aTable 
		addForeignKeyFrom: roleid 
		to: ((self tableNamed: 'ROLE') fieldNamed: 'ID')
		suffixExpression: 'ON DELETE CASCADE'.
! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mig 12/17/2007 01:01'!
tableForUSERROLE: aTable
	"USERROLE(USER_ID, ROLE_ID)"

	| userid roleid |
	
	userid := aTable createFieldNamed: 'USER_ID' type: (platform int4).
	roleid := aTable createFieldNamed: 'ROLE_ID' type: (platform int4).
		
	aTable 
		addForeignKeyFrom: userid 
		to: ((self tableNamed: 'USERS') fieldNamed: 'ID')
		suffixExpression: 'ON DELETE CASCADE'.
		
	aTable 
		addForeignKeyFrom: roleid 
		to: ((self tableNamed: 'ROLE') fieldNamed: 'ID')
		suffixExpression: 'ON DELETE CASCADE'.
! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mig 12/17/2007 01:02'!
tableForUSERS: aTable
	"USERS(ID, USERNAME, PASSWORDHASH)"
	
	(aTable createFieldNamed: 'ID' type: platform sequence) bePrimaryKey.
	aTable createFieldNamed: 'USERNAME' type: (platform varchar: 255).
	aTable createFieldNamed: 'PASSWORDHASH' type: (platform varchar: 255).! !
